


ARIBAS(1)                                               ARIBAS(1)


NAME
       aribas  - ARIBAS Interpreter for Arithmetic (Version 1.11)

SYNOPSIS
       aribas [ options ]  [ ari-file [ arg1 arg2 ...  ]]

DESCRIPTION
       This manpage describes the UNIX (and LINUX) version of ARIBAS.
       ARIBAS is an interactive Interpreter suitable for big integer
       arithmetic and multiprecision floating point arithmetic.  It
       has a syntax similar to Pascal or Modula-2, but contains also
       features from other programming languages like C, Lisp, Oberon.

USAGE
       The simplest way to use ARIBAS is as a calculator for (big
       integer) arithmetic. After starting ARIBAS, it will show it's
       prompt ==> and is ready to accept input. Simply enter the
       expression you want to calculate, followed by a full stop, and
       then press RETURN, for example

           ==> 123 + 456*789.

       ARIBAS answers

           -: 359907

       The symbol -: introduces the result.
       IMPORTANT.  To mark the end of your input, you must always type
       a full stop `.'  and then press RETURN.

       You can assign the result of a calculation to a variable, as in

           ==> F6 := 2**64 + 1.
           -: 18446_74407_37095_51617

       This calculates the 6th Fermat number (** denotes
       exponentiation) and assigns it to the variable F6 (note that
       ARIBAS is case sensitive, so this is not the same as f6).
       Later you can use this variable for example in the expression

           ==> 123**(F6 - 1) mod F6.
           -: 688_66214_58712_63971

       which shows (by Fermat's theorem) that F6 is not a prime
       number.
       The three most recent results are stored in the pseudo
       variables _, __, and ___. For example you can store the last
       result in the variable x by the command

           ==> x := _.
           -: 688_66214_58712_63971

       As you can see in the above examples, ARIBAS uses the
       underscore _ to structure the output of big integers (>=
       2**32). Also for input of integers you may use the underscore,
       the only condition is that immediately before and after the
       underscore there are digits, example:

           ==> z := 123_4567_890.
           -: 1234567890

       Here the output contains no underscore, which shows that z is
       less than 2**32.

       ARIBAS has several built-in functions for factorization, for
       example rho_factorize, which uses Pollard's rho algorithm.

           ==> rho_factorize(F6).

           working ..
           factor found after 512 iterations

           -: 274177

       To find the remaining cofactor, give the command

           ==> x := F6 div _.
           -: 6728_04213_10721

       To test whether this factor is prime, Rabin's probabilistic
       test rab_primetest can be applied:

           ==> rab_primetest(x).
           -: true

       The function rho_factorize is good for finding small factors
       (say up to 10 decimal digits); for more complicated
       factorization tasks a more powerful algorithm like the
       quadratic sieve qs_factorize should be used

           ==> qs_factorize(2**128+1).

       (Depending on the power of your computer, it will take a few
       seconds up to a few minutes to get a prime factor of the 7th
       Fermat number.)


   Control structures
       The for loop and the while loop in ARIBAS have a syntax as in
       MODULA-2. For example, the following command sequence
       calculates the factorial of 100.

           ==> x := 1;
               for i := 2 to 100 do
                   x := x*i;
               end;
               x.

       As you can see in this example, the input may extend over
       several lines.

       The above for loop is equivalent to the following while loop

           ==> x := 1; i := 2;
               while i <= 100 do
                   x := x*i;
                   inc(i);
               end;
               x.


       The branching construct
       if ...  then ...  elsif ...  else ...  end
       has also the same syntax as in MODULA-2.

   User defined functions
       The user can define his or her own functions. A typical example
       looks like

           ==> function fac(n: integer): integer;
               var
                   x,i: integer;
               begin
                   x := 1;
                   for i := 2 to n do
                       x := x*i;
                   end;
                   return x;
               end.

       If you have entered this correctly, ARIBAS echoes the function
       name
           -: fac
       and from now on you can use fac in the same way as a built-in
       function, e.g.

           ==> fac(32).
           -: 2_63130_83693_36935_30167_21801_21600_00000

       Note that inside function definitions all used variables must
       be explicitly declared, whereas on top level of the ARIBAS
       interpreter variables can be simply created by assignments.
       Here is another example, which shows some other data types
       supported by ARIBAS:

           ==> function sqrt_list(n: integer): array of real;
               var
                   vec: array[n] of real;
                   i: integer;
               begin
                   for i := 1 to n do
                       vec[i-1] := sqrt(i);
                   end;
                   return vec;
               end.

       This function returns an array of the square roots of the
       integers from 1 to n, for example

           ==> sqrt_list(10).
           -: (1.00000000, 1.41421356, 1.73205081, 2.00000000, 2.23606798,
           2.44948974, 2.64575131, 2.82842712, 3.00000000, 3.16227766)

       In a bigger programming project where you need several
       functions you would not enter them directly at the ARIBAS
       prompt but prepare the function definitions with an external
       text editor and save them in a file  with the extension .ari ,
       for example abcd.ari . This file can then be loaded by ARIBAS
       using the command

           ==> load("abcd").

       If there is a syntax error in the file, you get an error
       message of the form

           error in line <= 23 of loaded file
           if: end expected

       which tells you (in this example) that there is an error in the
       if construct in line 23 or earlier in the file. (Note that the
       error messages are sometimes not very precise.) You can then
       correct the error and load the file again.

   Online help
       The command

           ==> symbols(aribas).

       returns a list of all keywords and names of builtin functions
       of ARIBAS:

       (ARGV, _, __, ___, abs, alloc, and, arccos, arcsin, arctan,
       arctan2, aribas, array, atof, atoi, begin, binary, bit_and,
       bit_clear, bit_length, bit_not, bit_or, bit_set, bit_shift,
       bit_test, bit_xor, boolean, break, by, byte_length,
       byte_string, cardinal, cf_factorize, char, chr, close, concat,
       const, cos, dec, decode_float, div, do, double_float, else,
       elsif, end, even, exit, exp, extended_float, external,
       factor16, factorial, false, file, float, float_ecvt, floor,
       flush, for, frac, ftoa, function, gc, gcd, gcdx, get_filepos,
       get_floatprec, get_printbase, getenv, halt, help, if, inc,
       integer, isqrt, itoa, jacobi, length, load, log, long_float,
       make_unbound, max, max_arraysize, mem_and, mem_bclear,
       mem_bitswap, mem_bset, mem_btest, mem_byteswap, mem_not,
       mem_or, mem_shift, mem_xor, memavail, min, mod, mod_coshmult,
       mod_inverse, mod_pemult, new, nil, not, odd, of, open_append,
       open_read, open_write, or, ord, pi, pointer, prime32test,
       procedure, product, qs_factorize, rab_primetest, random,
       random_seed, read_block, read_byte, readln, real, record,
       return, rewind, rho_factorize, round, save_input, set_filepos,
       set_floatprec, set_printbase, short_float, sin, single_float,
       sort, sqrt, stack, stack2array, stack_empty, stack_pop,
       stack_push, stack_reset, stack_top, stderr, stdin, stdout,
       string, string_split, substr_index, sum, symbols, system, tan,
       then, timer, to, tolower, toupper, transcript, true, trunc,
       type, user, var, version, while, write, write_block,
       write_byte, writeln)

       For most of the symbols in this list, you can get a short
       online help using the function help(). For example, the command

           ==> help(gcd).

       gives an information on the function gcd (greatest common
       divisor), whereas

           ==> help(while).

       describes the syntax of the while loop. If you need more
       information than that contained in the online help, consult the
       documentation coming with the ARIBAS distribution.

   How to exit
       To end an ARIBAS session, type exit at the ARIBAS prompt

           ==> exit

       and then press the RETURN (ENTER) key.

       If you don't want to leave ARIBAS, but want to break out of an
       infinite loop or a calculation that lasts too long, type
       CONTROL-C (if you are running ARIBAS from within Emacs, you
       must press CONTROL-C twice). This will (in most cases) stop the
       current calculation and return to the ARIBAS prompt.

       Sometimes, while typing input, you commit an error. If you have
       already left the line containing the error you cannot correct
       it since you cannot return to that line. In this case you
       should simply type a full stop `.' , followed by RETURN. You
       will get an error message which you should ignore, and a new
       prompt ==> , allowing you to try again.

COMMAND LINE ARGUMENTS
       aribas [ options ]  [ ari-file [ arg1 arg2 ...  ]]


   options
       ARIBAS accepts several options at the command line. Some
       options (-q, -v, -b) are without arguments, other options (-c,
       -m, -h, -p) require an argument. Options requiring no argument
       may be merged, e.g. -qb is equivalent to -q -b. Arguments of
       options either follow immediately the option letter or are
       separated by white space, e.g. an argument 1000 to the -m
       option may be given either as -m1000 or -m 1000.
       Description of the options in detail:

       -q     (quiet mode) Suppresses all messages to the screen
              (version no, copyright notice, etc.) when ARIBAS is
              started.

       -v     (verbose mode, default) Does not suppress messages to
              the screen when ARIBAS is started.

       -c cols
              ARIBAS does its own line breaking when writing to the
              screen.  Normally it supposes that the screen (or the
              window in which ARIBAS runs) has 80 columns. With the -c
              option you can set another number, which must be between
              40 and 160 (in decimal representation).  For example, if
              you run ARIBAS in an Xterm window with 72 columns, use
              the option -c72 (or -c 72).

       -m mem
              Here mem is a number (in decimal representation) between
              64 and 16000. This number indicates how many Kilobytes
              of RAM ARIBAS should use for the ARIBAS heap. The
              default value depends on the options used when ARIBAS
              was compiled. Typically, under UNIX or LINUX it is 2
              Megabytes, corresponding to -m2000

       -h help-file-path
              The online help of ARIBAS depends on a file aribas.hlp,
              which should be in the range of the environment variable
              PATH. If this is not the case you can specify the exact
              path of the help file with the -h option. If for example
              the file aribas.hlp is in the directory /usr/local/lib,
              use the option -h /usr/local/lib The -h option can also
              be used if the help file has a different name. If the
              help file is named help-aribas and lies in the directory
              /home/joe/ari, use -h/home/joe/ari/help-aribas.

       -p ari-search-path
              With this option you can specify a search path for
              loading files with ARIBAS source code.  ari-search-path
              may be either the (absolute) pathname of one directory
              or several pathnames separated by colons. The user's
              home directory may be abbreviated by ~/.  Suppose that
              you have called ARIBAS with the option
              -p/usr/local/lib/aribas:~/ari/examples
              and that your home directory is /home/alice/.  Then the
              command

                      ==> load("factor").

              will search the file factor.ari first in the current
              directory, then in the directory /usr/local/lib/aribas
              and finally in /home/alice/ari/examples.

       -b     Batch mode when loading an ARIBAS source code file from
              the command line, see below.

   Further command line arguments
       ari-file [arg1 arg2 ...]
              The next command line argument after the options is
              interpreted as the name of a file with ARIBAS source
              code. If the file name has the extension .ari, this
              extension may be omitted. The file is loaded as if the
              command load("ari-file") had been given after the start
              of ARIBAS at the ARIBAS prompt. If the file is not found
              in the current directory it is searched in the
              directories specified by the -p option. If the batch
              mode option -b was given, the file is loaded and
              executed.  Afterwards ARIBAS exits without showing it's
              prompt. If the file cannot be loaded completely because
              of an error, ARIBAS exits immediately after the error
              message.  Without the -b option, ARIBAS enters
              interactive mode after loading the file by showing it's
              prompt ==> , and is ready for input.
              When further command line arguments arg1 arg2 ...
              follow ari-file, they are collected (as strings)
              together with ari-file in the vector ARGV which can be
              accessed from within ARIBAS.
              Example: If you call ARIBAS with the command line

                  aribas startup 4536 eisenstein

              and the current directory contains the file startup.ari,
              then ARIBAS loads it and the vector ARGV has the form

                  ==> ARGV.
                  -: ("startup", "4536", "eisenstein")

              If you need some arguments as numbers and not as
              strings, you can transform them by atoi (or atof); in
              our example

                  ==> x := atoi(ARGV[1]).
                  -: 4536

              will do it. The length of the vector ARGV can be
              determined by length(ARGV).

CONFIGURATION FILE
       Options for running ARIBAS can be specified also using a
       configuration file named .arirc .  ARIBAS searches for a
       configuration file in the following order:

           1) current directory
           2) home directory of the user

       There is a third possibility: You can define an environment
       variable ARIRC containing the name of the configuration file
       (which may be different from .arirc) including the full path.
       In the configuration file you can specify all command line
       options described above which begin with a - sign, however a
       separate line must be used for every single option. Lines
       beginning with the character # or empty lines are ignored.  In
       addition to the options described above, the configuration file
       may contain ARIBAS source code. For this purpose there must be
       a line reading

       -init

       Then everything after this line is treated as ARIBAS source
       code and executed when ARIBAS is started.
       The existence of a configuration file for ARIBAS does not
       exclude the possibility to give command line arguments. If an
       option (e.g. the -m option) is specified both in the
       configuration file and the command line but with different
       values, then the specification at the command line is valid.
       Analogously, a -v option on the command line overrides a -q
       option in the configuration file.  If there is -init code in
       the configuration file and an ari-file argument at the command
       line, then the -init code is executed first and afterwards the
       ari-file is loaded and its code executed.

AUTHOR
       ARIBAS was written by

           Otto Forster
           Math. Institut der LMU
           Theresienstr. 39
           D-80333 Muenchen, Germany

           Email   forster@rz.mathematik.uni-muenchen.de
           WWW     http://www.mathematik.uni-muenchen.de/~forster

FTP-SITE
       The latest version of ARIBAS can be obtained via anonymous ftp
       from
           ftp.mathematik.uni-muenchen.de
       directory
           pub/forster/aribas

BUGS
       Bug reports should be sent by email to

           forster@rz.mathematik.uni-muenchen.de


